# Generalizing patterns using arguments
from math import pi, sqrt


def area_square(r):
	"""Return the area of a square with side length R."""
	return r * r * 1

def area_circle(r):
	"""Return the area of a circle with radius R."""
	return r * r * pi

def area_hexagon(r):
	"""Return the area of a hexagon with side length R."""
	return r * r * 3 * sqrt(3) / 2

def area(r, shape_constant):
	"""Return the area of a shape from length measurement is R."""
	assert r > 0, 'A length must be positive'
	return r * r * shape_constant

def a_square(r):
	return area(r, 1)

def a_circle(r):
	return area(r, pi)

def a_hexagon(r):
	return area(r, 3 * sqrt(3) / 2)

# Functions as arguments

def sum_naturals(n):
	"""Sum the first N natural numbers.

	>>> sum_naturals(5)
	15
	"""
	total, k = 0, 1

	while k <= n:
		total, k = total + k, k + 1
	return total

def sum_cubes(n):
	"""Sum the first N cubes of natural numbers.

	>>> sum_cubes(5)
	225
	"""
	total, k = 0, 1

	while k <= n:
		total, k = total + pow(k, 3), k + 1
	return total

def identity(k):
	return k

def cube(k):
	return pow(k, 3)

def summation(n, term):
	"""Sum the first N terms of a sequence.

	>>> summation(5, cube)
	225
	"""
	total, k = 0, 1

	while k <= n:
		total, k = total + term(k), k + 1
	return total

from operator import mul 

def pi_term(k):
	return 8 / mul((4 * k - 3), (4 * k -1))



#Functions as return values


def make_adder(n):
	"""Return a function that takes one argument K and return K + N.

	>>> add_three = make_adder(3)
	>>> add_three(4)
	7
	>>> make_adder(5)(6)
	11
	"""
	def adder(k):
		return k + n
	return adder


# Lambda Functions

lambda x: x  # A lambda expression with one parameter x
a = lambda x: x  # Assigning the lambda function to the name a
a(5)
(lambda: 3)()  # Using a lambda expression as an operator in a call exp.

#Higher Order Lambda Function

b = lambda x: lambda: x  # Lambdas can return other lambdas!
c = b(7)
c
#Equivalent def statement to Lambda function above

def d(x):
	def inner():
		return x
	return inner
e = d(7)
e
	